# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing_extensions import Literal

__all__ = ["AccountsType"]

AccountsType = Literal["external_accounts", "internal_accounts"]
